import jieba


file_encoding = 'UTF-8'

if __name__ == '__main__':
    jieba.suggest_freq('沙瑞金', True)
    jieba.suggest_freq('易学习', True)
    jieba.suggest_freq('王大路', True)
    jieba.suggest_freq('京州', True)
    with open('../resources/chinese/nlp_test0.txt', encoding='UTF-8') as f:
        document = f.read()
        document_cut = jieba.cut(document)
        # print(jieba_cut)
        result = ' '.join(document_cut)
        result.encode('utf-8')
        with open('../resources/chinese/nlp_test1.txt', mode='w') as f2:
            f2.write(result)
    f.close()
    f2.close()
    with open('../resources/chinese/nlp_test2.txt', encoding='UTF-8') as f:
        document = f.read()
        document_cut = jieba.cut(document)
        # print(jieba_cut)
        result = ' '.join(document_cut)
        result.encode('utf-8')
        with open('../resources/chinese/nlp_test3.txt', mode='w') as f2:
            f2.write(result)
    f.close()
    f2.close()

    # 引入停用词
    stpwrdpath = '../resources/stop_words/stop_words.txt'
    stpwrd_dic = open(stpwrdpath, 'rb')
    stpwrd_content = stpwrd_dic.read()
    # 将停用此表转为List
    stpwrdlst = stpwrd_content.splitlines()
    stpwrd_dic.close()

    with open('../resources/chinese/nlp_test1.txt') as f3:
        res1 = f3.read()
    print(res1)
    with open('../resources/chinese/nlp_test3.txt') as f4:
        res2 = f4.read()
    print(res2)

    from sklearn.feature_extractiont.text import TfidVectorizer
    corpus = [res1, res2]
    vector = TfidVectorizer(stop_words=stpwrdlst)
    tfidf = vector.fit_transform(corpus)
    print(tfidf)
    # from jpype import *
    #
    # startJVM(getDefaultJVMPath(), "-ea")
    # java.lang.System.out.println("Hello World")
    # shutdownJVM()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug  1 18:31:11 2018

@author: luogan
"""

import jieba
import re
from gensim.models import word2vec
import multiprocessing
import gensim
import numpy as  np
import pandas as pd
import collections
import pandas


def segment_text(source_corpus, train_corpus, coding, punctuation):
    '''
    切词,去除标点符号
    :param source_corpus: 原始语料
    :param train_corpus: 切词语料
    :param coding: 文件编码
    :param punctuation: 去除的标点符号
    :return:
    '''
    with open(source_corpus, 'r', encoding=coding) as f, open(train_corpus, 'w', encoding=coding) as w:
        for line in f:
            # 去除标点符号
            line = re.sub('[{0}]+'.format(punctuation), '', line.strip())
            # 切词
            words = jieba.cut(line)
            w.write(' '.join(words))


# if __name__ == '__main__':


# 严格限制标点符号
strict_punctuation = '。，、＇：∶；?‘’“”〝〞ˆˇ﹕︰﹔﹖﹑·¨….¸;！´？！～—ˉ｜‖＂〃｀@﹫¡¿﹏﹋﹌︴々﹟#﹩$﹠&﹪%*﹡﹢﹦﹤‐￣¯―﹨ˆ˜﹍﹎+=<­­＿_-\ˇ~﹉﹊（）〈〉‹›﹛﹜『』〖〗［］《》〔〕{}「」【】︵︷︿︹︽_﹁﹃︻︶︸﹀︺︾ˉ﹂﹄︼'
# 简单限制标点符号
simple_punctuation = '’!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~'
# 去除标点符号
punctuation = simple_punctuation + strict_punctuation

# 文件编码
coding = 'utf-8'

# coding ="gb18030"
# 原始语料
source_corpus_text = 'santi.txt'

# 是每个词的向量维度
size = 10
# 是词向量训练时的上下文扫描窗口大小，窗口为5就是考虑前5个词和后5个词
window = 5
# 设置最低频率，默认是5，如果一个词语在文档中出现的次数小于5，那么就会丢弃
min_count = 1
# 是训练的进程数，默认是当前运行机器的处理器核数。
workers = multiprocessing.cpu_count()
# 切词语料
train_corpus_text = 'words.txt'
# w2v模型文件
model_text = 'w2v_size_{0}.model'.format(size)

# 切词 @TODO 切词后注释
segment_text(source_corpus_text, train_corpus_text, coding, punctuation)

# w2v训练模型 @TODO 训练后注释
sentences = word2vec.Text8Corpus(train_corpus_text)
model = word2vec.Word2Vec(sentences=sentences, size=size, window=window, min_count=min_count, workers=workers)
model.save(model_text)

# 加载模型
model = gensim.models.Word2Vec.load(model_text)

g = open("words.txt", "r")  # 设置文件对象
std = g.read()  # 将txt文件的所有内容读入到字符串str中
g.close()  # 将文件关闭

cc = std.split(' ')

dd = []
kkl = dict()

'''
将每个词语向量化，并且append 在dd中，形成一个二维数组
并形成一个字典，index是序号，值是汉字
'''
for p in range(len(cc)):
    hk = cc[p]
    if hk in model:
        vec = list(model.wv[hk])
        dd.append(vec)
        kkl[p] = hk

# 将二维数组转化成numpy

dd1 = np.array(dd)

from sklearn.cluster import KMeans

estimator = KMeans(n_clusters=100)  # 构造聚类器
estimator.fit(dd1)  # 聚类
label_pred = estimator.labels_  # 获取聚类标签

# index 是某条向量的序号，值是分类号
index1 = list(range(len(dd1)))
vc = pd.Series(label_pred, index=index1)

aa = collections.Counter(label_pred)
v = pandas.Series(aa)
v1 = v.sort_values(ascending=False)

for n in range(10):
    vc1 = vc[vc == v1.index[n]]
    vindex = list(vc1.index)

    kkp = pd.Series(kkl)

    print('第', n, '类的前10个数据')

    ffg = kkp[vindex][:10]
    ffg1 = list(set(ffg))

    print(ffg1)



texts = [['human', 'interface', 'computer'],
['survey', 'user', 'computer', 'system', 'response', 'time'],
['eps', 'user', 'interface', 'system'],
['system', 'human', 'system', 'eps'],
['user', 'response', 'time'],
['trees'],
['graph', 'trees'],
['graph', 'minors', 'trees'],
['graph', 'minors', 'survey']]

from gensim import corpora
dictionary = corpora.Dictionary(texts)
corpus = [dictionary.doc2bow(text) for text in texts]
print(corpus[0]) # [(0, 1), (1, 1), (2, 1)]


from gensim import models
tfidf = models.TfidfModel(corpus)

doc_bow = [(0,1), (1,1)]
print(tfidf[doc_bow])

tfidf.save("./model.tfidf")
tfidf = models.TfidfModel.load("./model.tfidf")




import scrapy
from scrapy import Request
from scrapy_redis.spiders import RedisSpider

from crawler.items import HTMLItem
from crawler.spiders.config import keywords
from urllib.parse import urlencode
from urllib.parse import urljoin
import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


class BaiduNewsSpider(RedisSpider):

    name = "BaiduNewsSpider"
    BASE_URL = "https://www.baidu.com/s?ie=utf-8&medium=0&rtt=1&bsst=1&tn=news&rsv_sug3=1&rsv_sug4=111&" \
               "rsv_sug1=1&f=3&rsp=1"

    request_params = {
        "ie": "utf-8",
        "cl": 2,
        "medium": 0,
        "rtt": 1,
        "bsst": 1,
        "tn": "news",
        "word": "贵阳",
        "rsv_sug3": 1,
        "rsv_sug4": 111,
        "rsv_sug1": 1,
        "f": 3,
        "rsp": 1,
        "rsv_dl": "news_b_pn",  # 首次有，但第二次变化，以后不变
        "x_bfe_rqs": "03E80",   # 首次没有，第二次有，以后不变
        "x_bfe_tjscore": "0.002032",  # 首次无，第二次有，以后变化
        "tngroupname": "organic_news",  # 首次无，以后有，第二次以后不变
        "pn": 0,  # 首次无，第二次有，以后变化 => 页面第一条结果在结果中的偏移量
    }

    def start_requests(self):
        for starting_url in self.get_starting_urls():
            yield Request(url=starting_url, callback=self.parse_search_page)

    def parse_search_page(self, response):
        for req in self.crawl_items(response):
            yield req
        for req in self.crawl_search_pages(response):
            yield req

    def crawl_items(self, response):
        links = response.xpath('//*[@id="content_left"]//a/@href')
        for link in links:
            yield response.follow(url=link, callback=self.download_page)

    def crawl_search_pages(self, response):
        links = response.xpath('//*[@id="page"]//a/@href')
        for link in links:
            yield response.follow(url=link, callback=self.crawl_items)

    def download_page(self, response):
        item = HTMLItem()
        text = response.xpath('//html/text()').getall()
        item["url"] = response.url
        item["text"] = text
        yield item

    def get_starting_urls(self):
        base_url = BaiduNewsSpider.BASE_URL
        # 主关键词
        main_words = keywords["main"]
        # 副关键词
        second_words = keywords["second"]
        # 组合每一个主关键词
        for mw in main_words:
            for sw in second_words:
                starting_url = base_url + "&" + urlencode({"word": mw + "+" + sw})
                yield starting_url



# -*- coding: utf-8 -*-

# Scrapy settings for crawler project
#
# For simplicity, this file contains only settings considered important or
# commonly used. You can find more settings consulting the documentation:
#
#     https://docs.scrapy.org/en/latest/topics/settings.html
#     https://docs.scrapy.org/en/latest/topics/downloader-middleware.html
#     https://docs.scrapy.org/en/latest/topics/spider-middleware.html

BOT_NAME = 'crawler'

SPIDER_MODULES = ['crawler.spiders']
NEWSPIDER_MODULE = 'crawler.spiders'


# Crawl responsibly by identifying yourself (and your website) on the user-agent
#USER_AGENT = 'crawler (+http://www.yourdomain.com)'

# Obey robots.txt rules
ROBOTSTXT_OBEY = False

# Configure maximum concurrent requests performed by Scrapy (default: 16)
#CONCURRENT_REQUESTS = 32

# Configure a delay for requests for the same website (default: 0)
# See https://docs.scrapy.org/en/latest/topics/settings.html#download-delay
# See also autothrottle settings and docs
#DOWNLOAD_DELAY = 3
# The download delay setting will honor only one of:
#CONCURRENT_REQUESTS_PER_DOMAIN = 16
#CONCURRENT_REQUESTS_PER_IP = 16

# Disable cookies (enabled by default)
#COOKIES_ENABLED = False

# Disable Telnet Console (enabled by default)
#TELNETCONSOLE_ENABLED = False

# Override the default request headers:
#DEFAULT_REQUEST_HEADERS = {
#   'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
#   'Accept-Language': 'en',
#}

# Enable or disable spider middlewares
# See https://docs.scrapy.org/en/latest/topics/spider-middleware.html
#SPIDER_MIDDLEWARES = {
#    'crawler.middlewares.PublicOpinionCrawlerSpiderMiddleware': 543,
#}

# Enable or disable downloader middlewares
# See https://docs.scrapy.org/en/latest/topics/downloader-middleware.html
DOWNLOADER_MIDDLEWARES = {
    'crawler.middlewares.UAPOOLS': 543,
    # 'crawler.middlewares.PublicOpinionCrawlerDownloaderMiddleware': 543,
    # 'crawler.middlewares.SeleniumDownloaderMiddleware': 543,
}

# Enable or disable extensions
# See https://docs.scrapy.org/en/latest/topics/extensions.html
#EXTENSIONS = {
#    'scrapy.extensions.telnet.TelnetConsole': None,
#}

# Configure item pipelines
# See https://docs.scrapy.org/en/latest/topics/item-pipeline.html
ITEM_PIPELINES = {
    # 'crawler.pipelines.PublicOpinionCrawlerPipeline': 300,
    'crawler.pipelines.MetaItemPipeline': 300,
}

# Enable and configure the AutoThrottle extension (disabled by default)
# See https://docs.scrapy.org/en/latest/topics/autothrottle.html
#AUTOTHROTTLE_ENABLED = True
# The initial download delay
#AUTOTHROTTLE_START_DELAY = 5
# The maximum download delay to be set in case of high latencies
#AUTOTHROTTLE_MAX_DELAY = 60
# The average number of requests Scrapy should be sending in parallel to
# each remote server
#AUTOTHROTTLE_TARGET_CONCURRENCY = 1.0
# Enable showing throttling stats for every response received:
#AUTOTHROTTLE_DEBUG = False

# Enable and configure HTTP caching (disabled by default)
# See https://docs.scrapy.org/en/latest/topics/downloader-middleware.html#httpcache-middleware-settings
HTTPCACHE_ENABLED = True
HTTPCACHE_EXPIRATION_SECS = 0
HTTPCACHE_DIR = 'httpcache'
HTTPCACHE_IGNORE_HTTP_CODES = []
HTTPCACHE_STORAGE = 'scrapy.extensions.httpcache.FilesystemCacheStorage'


# Settings for Scrapy
DOWNLOAD_DELAY = 0


# 日志
LOG_FILE = 'target/output/log/spider.log'
LOG_LEVEL = "DEBUG"

# Web Driver
WEB_DRIVER = "D:\\Program Files\\Google\\ChromeDriver\\chromedriver_win32\\76\\chromedriver.exe"


# scrapy-redis
# 1. 增加了一个去重容器类的配置, 作用使用Redis的set集合来存储请求的指纹数据, 从而实现请求去重的持久化
# DUPEFILTER_CLASS = "scrapy_redis.dupefilter.RFPDupeFilter"

# 2. 增加了调度的配置, 作用: 把请求对象存储到Redis数据, 从而实现请求的持久化.
# SCHEDULER = "scrapy_redis.scheduler.Scheduler"

# 3. 配置调度器是否要持久化, 也就是当爬虫结束了, 要不要清空Redis中请求队列和去重指纹的set。如果是True, 就表示要持久化存储, 就不清空数据, 否则清空数据
# SCHEDULER_PERSIST = True

# 4.
# REDIS_URL = "redis://127.0.0.1:6379/0"



# -*- coding: utf-8 -*-

# Define here the models for your scraped items
#
# See documentation in:
# https://docs.scrapy.org/en/latest/topics/items.html

import scrapy


class PublicOpinionCrawlerItem(scrapy.Item):
    # define the fields for your item here like:
    # name = scrapy.Field()
    pass


class HTMLItem(scrapy.Item):
    url = scrapy.Field()
    text = scrapy.Field()


class NewsItem(scrapy.Item):
    title = scrapy.Field()
    text = scrapy.Field()
    datetime = scrapy.Field()


class QQNewsItem(scrapy.Item):
    meta = scrapy.Field()


class QQNewsJRYWItem(scrapy.Item):
    pass


from collections import namedtuple as nt, namedtuple

Article_Fields = ['id', 'title', 'article_type', 'body', 'release_time', 'create_by',
                  'create_time', 'update_by', 'update_time']
Article = namedtuple("Article", Article_Fields)

Comment_Fields = ['id', 'article_id', 'comment', 'create_by', 'create_time', 'update_by', 'update_time']
Comment = nt("Comment", Comment_Fields)


# utilities for request and response in scrapy.
import json
import logging
import re

xpath_all_links = "//a/@href"


class ResponseUtils:
    @staticmethod
    def get_all_links(response):
        return response.xpath(xpath_all_links).getall()

    @staticmethod
    def jsonp_to_json(jsonp):
        if jsonp is not None:
            try:
                a = re.search(r'[^\(]*\((.*)\)$', jsonp).group(1)
                return json.loads(a)
            except Exception as e:
                err_msg = "JSONP 转换错误"
                print(err_msg, e)
                logging.error(err_msg, e)



# make it work in both python2 both python3
import json
import logging
import sys
import time

from datetime import datetime

from crawler.nlpapi import config

IS_PY3 = sys.version_info.major == 3
if IS_PY3:
    from urllib.request import urlopen
    from urllib.request import Request
    from urllib.error import URLError
    from urllib.parse import urlencode
    from urllib.parse import quote_plus
else:
    import urllib2
    from urllib import quote_plus
    from urllib2 import urlopen
    from urllib2 import Request
    from urllib2 import URLError
    from urllib import urlencode

# skip https auth
# @TODO: 可能不安全
from crawler.nlpapi.config import ACCOUNTS, AIP_SLEEP_TIME, RETRY_TIMES
import ssl
ssl._create_default_https_context = ssl._create_unverified_context


class Client:
    """  TOKEN start """
    TOKEN_URL = 'https://aip.baidubce.com/oauth/2.0/token'

    def __init__(self):
        self.next = 0
        self.accounts = ACCOUNTS

    def get_aip_config(self):
        """轮询方式获取一个百度接口的配置
        :return:
        """
        pos = self.next
        accounts = self.accounts
        self.next = (pos + 1) % len(accounts)
        cfg = accounts[pos]
        if not hasattr(cfg, "grant_type") or "client_credentials" not in cfg.values():
            cfg["grant_type"] = "client_credentials"
        return {
            "grant_type": cfg["grant_type"],
            "client_id": cfg["api_key"],
            "client_secret": cfg["secret_key"]
        }

    def fetch_token(self):
        """  Fetch Token """
        params = self.get_aip_config()
        post_data = urlencode(params)
        if IS_PY3:
            post_data = post_data.encode('utf-8')
        req = Request(Client.TOKEN_URL, post_data)
        try:
            f = urlopen(req, timeout=5)
            result_str = f.read()
        except URLError as err:
            print(err)
        if IS_PY3:
            result_str = result_str.decode()

        result = json.loads(result_str)

        if 'access_token' in result.keys() and 'scope' in result.keys():
            if 'brain_all_scope' not in result['scope'].split(' '):
                print('please ensure has check the  ability')
                exit()
            return result['access_token']
        else:
            print('please overwrite the correct API_KEY and SECRET_KEY')
            exit()

    def request(self, url, data={}):
        """调用百度远程服务
        :param url:
        :param data:
        :return: unicode字符串
        """
        req = Request(url, json.dumps(data).encode())
        has_error = False
        try:
            f = urlopen(req)
            result_str = f.read()
            if IS_PY3:
                result_str = result_str.decode()
            return result_str
        except URLError as err:
            print(err)
            logging.error(err)


class Aip:
    TOPIC_URL = "https://aip.baidubce.com/rpc/2.0/nlp/v1/topic"
    SENTIMENT_CLASSIFY_URL = "https://aip.baidubce.com/rpc/2.0/nlp/v1/sentiment_classify"

    def __init__(self):
        self.client = Client()
        self.token = None
        self.last_fetch_time = datetime.fromtimestamp(0)

    def _request_raw(self, url, data):
        client = self.client
        if AIP_SLEEP_TIME:
            time.sleep(AIP_SLEEP_TIME)
        response = client.request(url, data)
        # 有时会超时打不开，所以用空对象代替
        return response if response else "{}"

    def _request(self, url, data):
        return json.loads(self._request_raw(url, data))

    def _check_token(self):
        """是否过期"""
        interval = datetime.now() - self.last_fetch_time
        if interval.days > 29:
            return True

    def _fetch_token(self):
        if self._check_token():
            self.token = self.client.fetch_token()
        return self.token

    def _truncate(self, text, max_size, encoding="UTF-8"):
        while len(bytes(text, encoding=encoding)) > max_size:
            text = text[:len(text)-1]
        return text

    def _get_available_url(self, url, encoding="UTF-8"):
        token = self._fetch_token()
        available_url = url+"?charset="+encoding+"&access_token="+token
        return available_url

    def topic(self, title, content):
        """文章分类接口
        参见：https://ai.baidu.com/docs#/NLP-Apply-API/e3191e88
        :param title:
        :param content:
        :return:
        """
        try:
            data = {
                "title": self._truncate(title, max_size=80),
                "content": self._truncate(content, max_size=65535),
            }
            url = self._get_available_url(self.TOPIC_URL)
            resp = self._request_raw(url, data)
            retry_times = 0
            while not resp and retry_times < RETRY_TIMES:
                resp = self._request_raw()
                retry_times += 1
            if not resp:
                logging.error("百度请求接口数据返回异常")
                print("百度请求接口数据返回异常")
            return json.loads(resp if resp else {})
        except (TypeError, Exception) as e:
            logging.error(e)
            print(e)

    def sentiment_classify(self, text):
        """
        情感分析接口
        参见：https://ai.baidu.com/docs#/NLP-Apply-API/955c17f6
        :param text: less than 2048 bytes
        :return:
        """
        data = {
            "text": self._truncate(text, max_size=2048)
        }
        url = self._get_available_url(self.SENTIMENT_CLASSIFY_URL)
        return self._request(url, data)


class Constants(object):
    SENTIMENT_NEGATIVE = 0
    SENTIMENT_NORMAL = 1
    SENTIMENT_POSITIVE = 2
    TOPIC_CLASS = [
        "国际", "体育", "娱乐", "社会", "财经", "时事", "科技", "情感", "汽车", "教育", "时尚", "游戏", "军事", "旅游",
        "美食", "文化", "健康养生", "搞笑", "家居", "动漫", "宠物", "母婴育儿", "星座运势", "历史", "音乐", "综合",
    ]


if __name__ == '__main__':
    # client = Client()
    start = time.time()
    # for i in range(100):
    #     client.fetch_token()
    aip = Aip()
    text = "".join(["a" for i in range(2049)])
    a = aip.sentiment_classify(text=text)
    print(a)
    end = time.time()
    print(end - start)



import re

import scrapy as scrapy
from scrapy import Request
from scrapy_redis.spiders import RedisSpider


class PeopleDaily(RedisSpider):
    name = "PeopleDailySpider"

    start_urls = ["http://www.people.com.cn/"]

    def start_requests(self):
        for url in PeopleDaily.start_urls:
            yield Request(url=url, callback=self.parse0)

    def parse0(self, response):
        pass


# -*- coding: utf-8 -*-

# Define your item pipelines here
#
# Don't forget to add your pipeline to the ITEM_PIPELINES setting
# See: https://docs.scrapy.org/en/latest/topics/item-pipeline.html
import logging
from datetime import datetime

from crawler.model.tables import Article, Comment
from crawler.db import db
from crawler.items import HTMLItem, QQNewsItem


class PublicOpinionCrawlerPipeline(object):
    def process_item(self, item, spider):
        return item


class TextPipeline(object):
    count = 0

    def process_item(self, item, spider):
        if isinstance(item, HTMLItem):
            self.store_item(self.clean_item(item), spider)
        # else:
        # return DropItem()

    def store_item(self, item, spider):
        sql = "INSERT INTO `text`(`url`, `text`) VALUE(%s, %s)"
        conn = db.get_connection()
        cursor = conn.cursor()
        r = cursor.execute(sql, [item['url'], item['text']])
        conn.commit()
        TextPipeline.count += 1
        if r:
            print('=================Yes======', TextPipeline.count)
        conn.close()

    def clean_item(self, item):
        return item


class MetaItemPipeline(object):
    def process_item(self, item, spider):
        if isinstance(item, QQNewsItem):
            self.store_item(item, spider)

    def store_item(self, item, spider):
        try:
            meta = item["meta"]
            conn = db.get_connection()
            li = meta["list_info"]
            detail = meta["detail"]
            article = {
                "id": detail["article_id"],
                "title": li["title"],
                "body": detail["text"],
                "release_time": datetime.strptime(li["publish_time"], "%Y-%m-%d %H:%M:%S"),
                "create_by": li["source"],
                "create_time": datetime.strptime(li["publish_time"], "%Y-%m-%d %H:%M:%S"),
                "update_by": li["source"],
                "update_time": datetime.strptime(li["update_time"], "%Y-%m-%d %H:%M:%S"),
            }
            sql = "INSERT INTO `cadrem_pubopinion_article`(`id`,`title`,`article_type`," \
                  "`body`,`release_time`,`create_by`,`create_time`, `update_by`, `update_time`) " \
                  "VALUE(%(id)s, %(title)s, DEFAULT, %(body)s, %(release_time)s, %(create_by)s, %(create_time)s, " \
                  "%(update_by)s, %(update_time)s)"
            cursor = conn.cursor()
            self._delete_exists(conn, article["id"])
            cursor.execute(sql, article)
            comments = []
            sql = "INSERT INTO `cadrem_pubopinion_comment`(`id`,`article_id`,`comment`,`create_by`,\
                `create_time`,`update_by`,`update_time`) VALUES(DEFAULT, %(article_id)s, %(comment)s," \
                  "%(create_by)s, %(create_time)s, %(update_by)s, %(update_time)s)"
            for comment in meta["comments"]:
                for c in comment["data"]["oriCommList"]:
                    cc = {
                        "article_id": detail["article_id"],
                        "comment": c["content"],
                        "create_by": c["userid"],
                        "create_time": datetime.fromtimestamp(int(c["time"])),
                        "update_by": c["userid"],
                        "update_time": datetime.fromtimestamp(int(c["time"])),
                    }
                    comments.append(cc)
            cursor.executemany(sql, comments)
            cursor.close()
            conn.commit()
            TextPipeline.count += 1
        except Exception as e:
            print("数据库入库错误：", e)
            logging.error("数据库入库错误：", e)
            conn.close()

    def _delete_exists(self, connection, id):
        sql = "DELETE FROM cadrem_pubopinion_article WHERE id=%s "
        cursor = connection.cursor()
        result = cursor.execute(sql, id)
        sql = "DELETE FROM cadrem_pubopinion_comment WHERE article_id=%s"
        result2 = cursor.execute(sql, id)
        return result and result2



# -*- coding: utf-8 -*-

# Define here the models for your spider middleware
#
# See documentation in:
# https://docs.scrapy.org/en/latest/topics/spider-middleware.html
import logging
import random
import time

from scrapy import signals
from scrapy.downloadermiddlewares.useragent import UserAgentMiddleware
from scrapy.http import HtmlResponse
from selenium import webdriver
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.wait import WebDriverWait

from crawler.settings import WEB_DRIVER


class PublicOpinionCrawlerSpiderMiddleware(object):
    # Not all methods need to be defined. If a method is not defined,
    # scrapy acts as if the spider middleware does not modify the
    # passed objects.

    @classmethod
    def from_crawler(cls, crawler):
        # This method is used by Scrapy to create your spiders.
        s = cls()
        crawler.signals.connect(s.spider_opened, signal=signals.spider_opened)
        return s

    def process_spider_input(self, response, spider):
        # Called for each response that goes through the spider
        # middleware and into the spider.

        # Should return None or raise an exception.
        return None

    def process_spider_output(self, response, result, spider):
        # Called with the results returned from the Spider, after
        # it has processed the response.

        # Must return an iterable of Request, dict or Item objects.
        for i in result:
            yield i

    def process_spider_exception(self, response, exception, spider):
        # Called when a spider or process_spider_input() method
        # (from other spider middleware) raises an exception.

        # Should return either None or an iterable of Request, dict
        # or Item objects.
        pass

    def process_start_requests(self, start_requests, spider):
        # Called with the start requests of the spider, and works
        # similarly to the process_spider_output() method, except
        # that it doesn’t have a response associated.

        # Must return only requests (not items).
        for r in start_requests:
            yield r

    def spider_opened(self, spider):
        spider.logger.info('Spider opened: %s' % spider.name)


class PublicOpinionCrawlerDownloaderMiddleware(object):
    # Not all methods need to be defined. If a method is not defined,
    # scrapy acts as if the downloader middleware does not modify the
    # passed objects.

    @classmethod
    def from_crawler(cls, crawler):
        # This method is used by Scrapy to create your spiders.
        s = cls()
        crawler.signals.connect(s.spider_opened, signal=signals.spider_opened)
        return s

    def process_request(self, request, spider):
        # Called for each request that goes through the downloader
        # middleware.

        # Must either:
        # - return None: continue processing this request
        # - or return a Response object
        # - or return a Request object
        # - or raise IgnoreRequest: process_exception() methods of
        #   installed downloader middleware will be called
        return None

    def process_response(self, request, response, spider):
        # Called with the response returned from the downloader.

        # Must either;
        # - return a Response object
        # - return a Request object
        # - or raise IgnoreRequest
        return response

    def process_exception(self, request, exception, spider):
        # Called when a download handler or a process_request()
        # (from other downloader middleware) raises an exception.

        # Must either:
        # - return None: continue processing this exception
        # - return a Response object: stops process_exception() chain
        # - return a Request object: stops process_exception() chain
        pass

    def spider_opened(self, spider):
        spider.logger.info('Spider opened: %s' % spider.name)


class UAPOOLS(UserAgentMiddleware):
    def __init__(self, user_agent=''):
        self.user_agent = user_agent

    def process_request(self, request, spider):
        '''使用代理UA，随机选用'''
        ua = random.choice(self.user_agent_pools)
        # print('当前使用的user-agent是' + ua)
        request.headers.setdefault('User-Agent', ua)

    user_agent_pools = [
        'Mozilla/5.0 (Windows NT 6.2) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1062.0 Safari/536.3',
        'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.3 (KHTML, like Gecko) Chrome/19.0.1063.0 Safari/536.3',
        'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/49.0.2623.75 Safari/537.36',
    ]


class SeleniumDownloaderMiddleware(object):
    def __init__(self, timeout=25):
        chrome_options = Options()
        prefs = {
            'profile.default_content_setting_values': {
                'images': 2,  # 禁用图片的加载
                # 'javascript': 2  # 禁用js，可能会导致通过js加载的互动数抓取失效
            }
        }
        chrome_options.add_experimental_option("prefs", prefs)
        chrome_options.add_argument('--headless')  # 使用无头谷歌浏览器模式
        chrome_options.add_argument('--disable-gpu')
        chrome_options.add_argument('--no-sandbox')
        self.browser = webdriver.Chrome(executable_path=WEB_DRIVER, chrome_options=chrome_options)
        self.timeout = timeout
        self.browser.maximize_window()
        # self.browser.implicitly_wait(20)
        # self.browser.set_page_load_timeout(25)
        self.browser.set_page_load_timeout(self.timeout)
        self.wait = WebDriverWait(self.browser, self.timeout)

    def __del__(self):
        # logging.debug("Chrome driver is Closing.")
        # self.browser.close()
        pass

    def process_request(self, request, spider):
        """
           用ChromeDriver抓取页面
           :param request: Request对象
           :param spider: Spider对象
           :return: HtmlResponse
           """
        logging.info('******ChromeDriver is Starting******')
        try:
            self.browser.get(request.url)
            # self.wait.until(EC.presence_of_element_located((By.XPATH, '//div[@class="s-result-list sg-row"]')))
            self.load_logic(request, spider)
            resp = HtmlResponse(url=request.url, body=self.browser.page_source, request=request, encoding='utf-8',
                            status=200)
            # self.browser.close()
            return resp
        except TimeoutException:
            return HtmlResponse(url=request.url, status=500, request=request)

    @classmethod
    def from_crawler(cls, crawler):
        s = cls()
        crawler.signals.connect(s.spider_opened, signal=signals.spider_opened)
        return s

    def spider_opened(self, spider):
        spider.logger.info('Spider opened: %s' % spider.name)

    def load_logic(self, request, spider):
        meta = request.meta
        if "selenium" in meta.keys():
            self._load_long_page(request, spider)

    def _load_long_page(self, request, spider):
        selenium = request.meta["selenium"]
        keys = selenium.keys()
        if "load_long_page" in keys and selenium["load_long_page"]:
            times = 5 # 次数
            interval = 2 # 秒
            if "load_long_page_times" in keys:
                times = selenium["load_long_page_times"]
            if "load_long_page_interval" in keys:
                interval = selenium["load_long_page_interval"]
            for i in range(times):
                self.browser.execute_script("window.scrollTo(0,100000)")
                time.sleep(interval)


from gensim import corpora
import jieba
documents = ['工业互联网平台的核心技术是什么',
            '工业现场生产过程优化场景有哪些']
def word_cut(doc):
    seg = [jieba.lcut(w) for w in doc]
    return seg

texts= word_cut(documents)

##为语料库中出现的所有单词分配了一个唯一的整数id
dictionary = corpora.Dictionary(texts)
dictionary.token2id
bow_corpus = [dictionary.doc2bow(text) for text in texts]
bow_corpus

from gensim import models
# train the model
tfidf = models.TfidfModel(bow_corpus)

def StopWordList(filepath):
    wlst = [w.strip() for w in open(filepath, 'r', encoding='utf-8').readlines()]
    return wlst

stop_flags = ['x', 'c', 'u','d', 'p', 't', 'uj', 'm', 'f', 'r']

def seg_sentence(sentence,stop_words):
    sentence_seged = jieba.cut(sentence.strip())
    # sentence_seged = set(sentence_seged)
    outstr = ''
    for word in sentence_seged:
        if word not in stop_words:
            if word != '\t':
                outstr += word
                outstr += ' '

    return outstr.split(' ')


 #1、将【文本集】生产【分词列表】
    texts = [seg_sentence(seg,stop_words) for seg in open(tpath,'r',encoding='utf8').readlines()]
#一、建立词袋模型
    #2、基于文件集建立【词典】，并提取词典特征数
    dictionary = corpora.Dictionary(texts)
    feature_cnt = len(dictionary.token2id.keys())
    #3、基于词典，将【分词列表集】转换为【稀疏向量集】，也就是【语料库】
    corpus = [dictionary.doc2bow(text) for text in texts]
#二、建立TF-IDF模型
    #4、使用“TF-TDF模型”处理【语料库】
    tfidf = models.TfidfModel(corpus)
#三构建一个query文本，利用词袋模型的字典将其映射到向量空间
    #5、同理，用词典把搜索词也转换为稀疏向量
    kw_vector = dictionary.doc2bow(seg_sentence(keyword,stop_words))
    #6、对稀疏向量建立索引
    index = similarities.SparseMatrixSimilarity(tfidf[corpus],num_features=feature_cnt)
    #7、相似的计算
    sim = index[tfidf[kw_vector]]



from gensim.models import word2vec
import logging

logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)

sentences = word2vec.Text8Corpus("text.txt")

model = word2vec.Word2Vec(sentences, sg=1, size=100, window=5, min_count=5, negative=3, sample=0.001, hs=1, workers=4)

model.save("text.model")

print(model["足球"])
print(model.most_similar("足球"))


import logging
from datetime import datetime

from analysis.dbutils import select, update_by_id, table_article, select_not_sentiment, select_not_topic
from crawler.db.db import get_connection
from crawler.nlpapi.core import TextAnalyzer, BaiduAnalyzer


def get_hot_spot(limit=20, start=datetime.min, end=datetime.max):
    """Get limit number of hot spots in the time between start and end.
    Of course, the function will select specified number of hot spots between start and end.
    By default, it will select hot spots of all in the database.
    :param limit: number of selecting hot spots, default is 20.
    :param start: start time, default is zero.
    :param end: deadline, default is datetime.max.
    :return: a list of hot spots, it may be less than num, if data is too lack in database.
    """
    try:
        conn = get_connection()
        cursor = conn.cursor()
        # For getting number of news in specified range of date.
        sql = """
            -- 统计最近一天之内，评论最多的新闻，新闻内容不可为空，如果为空则不显示。
            SELECT 
                COUNT(cpc.id)   AS hot_degree , 
                cpa.create_by   AS source, 
                cpa.title       AS title, 
                cpa.body        AS content, 
                cpa.create_time AS publish_time
            FROM  cadrem_pubopinion_comment AS cpc
            JOIN cadrem_pubopinion_article  AS cpa ON cpa.id = cpc.article_id 
            WHERE 
                cpa.create_time BETWEEN %(start)s AND %(end)s 
                -- AND (cpa.body <> NULL OR cpa.body <> "")
            GROUP BY cpa.id 
            ORDER BY hot_degree DESC 
            LIMIT 0, %(limit)s;
        """
        params = {
            "limit": limit,
            "start": start,
            "end": end,
        }
        success = cursor.execute(sql, params)
        if not success:
            raise Exception("查询未成功！")
        sets = cursor.fetchall()
        cursor.close()
        conn.close()
    except Exception as e:
        logging.error(e)
        print(e)
    finally:
        if conn is not None:
            conn.close()
    # make data model
    return [
        {"hot_degree": _0, "source": _1, "title": _2, "content": _3, "publish_time": _4}
        for _0, _1, _2, _3, _4 in sets
    ]


class ArticleHandle(object):

    def __init__(self):
        self.analyzer = BaiduAnalyzer()
        pass

    def update_sentiment(self, batch=10):
        affected = 0
        for i in self._sentiment_analyze(batch):
            print(i)
            affected += self._update_sentiment(i)
        return affected

    def _sentiment_analyze(self, batch):
        """
        Analyze sentiment
        :param batch:
        :return:
        """
        for rows in select_not_sentiment(table_article, batch=batch):
            texts = []
            for row in rows:
                text = {
                    "id": row[0],
                    "params": {
                        "text": row[0] + row[3]
                    }
                }
                texts.append(text)
            yield self.analyzer.sentiment_classify(*texts)

    def _update_sentiment(self, results):
        rows = []
        for result in results:
            if result["result"] and "error_code" not in result["result"].keys():
                rows.append({
                    "id": result["id"],
                    "sentiment": result["result"]["items"][0]["sentiment"]
                })
            else:
                logging.error(result)
                print(result)
        return update_by_id(*rows)

    def update_topic(self, batch=10):
        """
        Classify and update database
        :return:
        """
        affetcted_rows = 0
        for i in self._topic_classify(batch):
            affetcted_rows += self._update_topic_classify(i)
        return affetcted_rows

    def _topic_classify(self, batch=100):
        """
        Select has not been classified articles
        :param batch:
        :return: a list of tuples
        """
        for rows in select_not_topic(table=table_article, batch=batch):
            articles = []
            for row in rows:
                article = {
                    "id": row[0],
                    "params": {
                        "title": row[1],
                        "content": row[3]
                    }
                }
                articles.append(article)
            yield self.analyzer.topic(*articles)

    def _update_topic_classify(self, results):
        """
        :param results: [{"id": id, "result": result}, ...]
        :return:
        """
        rows = []
        for result in results:
            if result["result"] and "error_code" not in result["result"].keys():
                response_result = result["result"]
                lv1 = response_result["item"]["lv1_tag_list"]
                lv2 = response_result["item"]["lv2_tag_list"]
                types = {i["tag"] for i in lv1 if i["tag"] != "" or i["tag"] is not None}
                types.update({i["tag"] for i in lv2 if i["tag"] != "" or i["tag"] is not None})
                lv = "|".join(types)
                rows.append({
                    "id": result["id"],
                    "article_type": lv,
                })
            else:
                logging.error(result)
                print(result)
        return update_by_id(*rows)


if __name__ == '__main__':
    handle = ArticleHandle()
    # handle.topic()
    # print("end")
    #
    # for i in handle._topic_classify(1):
    #     print(i)
    #     result = handle._update_topic_classify(i)
    #     print(i)
    handle.update_sentiment(1)



# 启用的数据库配置
ENABLED_DB = "test00"
# ENABLED_DB = "localhost"
# ENABLED_DB = "remote_dev"


# 数据库的详细配置
DB_CONFIG = [
    {
        "name": "localhost",
        "hostname": "localhost",
        "username": "root",
        "password": "123",
        "database": "public_opinion",
        "pool_size": 5,
    },
    {
        "name": "product",
        "hostname": "",
        "username": "",
        "password": "",
        "database": "",
    },
    {
        "name": "remote_dev",
        "hostname": "swc.ddns.net",
        "username": "root",
        "password": "1234",
        "database": "",
        "pool_size": 5
    },
    {
        "name": "test",
        "hostname": "localhost",
        "username": "root",
        "password": "123",
        "database": "public_opinion_test",
        "pool_size": 5
    },
    {
        "name": "crawler",
        "hostname": "localhost",
        "username": "root",
        "password": "123",
        "database": "",
        "pool_size": 5
    },
    {
        "name": "target",
        "hostname": "localhost",
        "username": "root",
        "password": "123",
        "database": "",
        "pool_size": 5
    },
    {
        "name": "test00",
        "hostname": "localhost",
        "username": "root",
        "password": "123",
        "database": "test00",
        "pool_size": 5
    }
]



import json
import logging
import re
from urllib.parse import urlencode, parse_qs, urlparse, urljoin

import scrapy
from scrapy import Request
from scrapy_redis.spiders import RedisSpider

from crawler.items import QQNewsItem
from crawler.utility.request2response import ResponseUtils as ru


class QQNewsSpider(scrapy.Spider):
    name = "QQNewsSpider"
    urls = ["https://news.qq.com/"]
    re_news = r"(?i:https?://new\.qq\.com/.*(?=html))"
    xpath_all_links = "//a/@href"
    xpath_news_detail = "/html/body/div[3]/div[1]/div[1]/div[2]//text()"

    # 热点精选
    base_url = "https://pacaio.match.qq.com/irs/rcd"
    # 今日要闻
    jin_ri_yao_wen = "https://i.match.qq.com/ninja/fragcontent?pull_urls=news_top_2018&callback=__jp0"
    # 热点精选
    re_dian_jing_xuan = "https://pacaio.match.qq.com/irs/rcd?cid=137&id=&ext=top&page=0&expIds=&callback=__jp1"
    # 热门资讯 cid=4是热门资讯，cid=137是热点精选
    # 该链接token必须存在，应该根据index.js中的cid=xxx&token=去搜索除token
    re_men_zi_xvn = "https://pacaio.match.qq.com/irs/rcd?cid=4&token=9513f1a78a663e1d25b46a826f248c3c&ext=&page=0&expIds=&callback=__jp2"
    # index.js 用来获取token
    index_js_url = "https://mat1.gtimg.com/pingjs/ext2020/newom/build/static/js/index.js"
    # index.js
    index_js = ""
    tokens = []
    cids = ['137']
    # parameters template
    param_tmp0 = {
        "cid": 0,               # 数字
        "token": "",            # index.js 中，使用cid=xxx&token=找寻
        "id": "",               #
        "ext": "top",           # ext = 0 if cid == '137'
        "expIds": "",           # 某些链接下上一个链接中的所有新闻名字取日期+紧接6位字母的分割，例如：20190101ABCDEF|...
        "callback": "__jp1",    # 回调函数名称
    }
    re_news_url = r"(?i:https?://new\.qq\.com.*(?=\.html))"
    comments_url = r'https://coral.qq.com/article/4157351119/comment/v2?callback=_article4157351119commentv2&orinum=1&oriorder=o&pageflag=1&cursor=0&scorecursor=0&orirepnum=2&reporder=o&reppageflag=1&source=1&_=1568249359993'

    custom_settings = {

    }

    def __init__(self, *args, **kwargs):
        super(QQNewsSpider, self).__init__(*args, **kwargs)
        self.count = 0

    def start_requests(self):
        yield self.download_index_js()

    def download_index_js(self):
        index_js = QQNewsSpider.index_js_url
        return Request(url=index_js, callback=self.parse_index_js, dont_filter=True)

    def parse_index_js(self, response):
        js = QQNewsSpider.index_js = response.text
        QQNewsSpider.tokens = [
            {"cid": cid, "token": token}
            for x in re.findall(r'cid=[\d]*&token=[\w]+', js)
            for cid, token in [
                [y.split("=")[1] for y in x.split("&")]
            ]
        ]
        # for req in self.download_jin_ri_yao_wen(response):
        #     yield req
        for req in self.download_channels(response, QQNewsSpider.cids):
            yield req
        # print("===================================")
        # for req in self.test_download_comments(response):
        #     print(req)
        #     yield req

    def download_jin_ri_yao_wen(self, response):
        yield response.follow(QQNewsSpider.jin_ri_yao_wen, callback=self.parse_jin_ri_yao_wen, dont_filter=True)

    def parse_jin_ri_yao_wen(self, response):
        logging.debug("======解析《今日要闻》=======")
        items = ru.jsonp_to_json(response.text)
        for item in items:
            meta = dict(response.meta)
            meta["list_info"] = item
            url = item["url"]
            if self._match_new_url(url):
                yield response.follow(url, meta=meta, callback=self.parse_news_detail)

    def parse_news_detail(self, response):
        meta = response.meta
        text = response.text
        pattern = r"(?i:\s*window.DATA\s*=\s*(\{*(\n.*)+?\s+)(?=</script>))"
        match = re.search(pattern, text)
        if match:
            text = match.group(1)
            detail = json.loads(text)
            detail["text"] = "".join(response.xpath("/html/body/div[3]/div[1]/div[1]/div[2]//p/text()").getall())
            meta["detail"] = detail
            for req in self.download_comments(response):
                yield req

    def download_comments(self, response):
        logging.debug("=========================================================")
        logging.debug("====================爬取评论===============================")
        meta = response.meta
        params = {'orinum': '30', 'oriorder': 'o', 'pageflag': '1',
                  'cursor': '0', 'scorecursor': '0', 'orirepnum': '30',
                  'reporder': 'o', 'reppageflag': '1',
                  'source': '1', '_': '1568249359993'}
        detail = meta["detail"]
        comment_id = detail["comment_id"]
        params["callback"] = '_article%scommentv2' % comment_id
        base_url = "https://coral.qq.com/article/%s/comment/v2"
        url = (base_url + "?" + urlencode(params)) % comment_id
        meta["comments"] = []
        yield response.follow(url, meta=meta, callback=self.download_comments_remaining)

    def download_comments_remaining(self, response):
        logging.debug("==========================爬取剩余的评论==========================")
        meta = response.meta
        data = ru.jsonp_to_json(response.text)
        meta["comments"].append(data)
        if "oriCommList" in data["data"].keys() and len(data["data"]["oriCommList"]) < 30:
            yield self.extract_item(meta)
        else:
            qs = parse_qs(urlparse(response.url)[4])
            qs["cursor"] = data["data"]["last"]
            url = urljoin(response.url, "?" + urlencode(qs, True))
            yield response.follow(url, meta=meta, callback=self.download_comments_remaining)

    def extract_item(self, meta):
        item = QQNewsItem()
        item["meta"] = meta
        return item

    def download_re_dian_jing_xuan(self, response, cid):
        meta = response.meta
        if "params" not in meta.keys():
            params = dict(QQNewsSpider.param_tmp0)
            tokens = QQNewsSpider.tokens
            params["cid"] = cid
            params["token"] = [item["token"] for item in tokens if item["cid"] == params['cid']][0]
            params["ext"] = "top"
            params["callback"] = "__jp1"
            params["page"] = 0
        else:
            params = meta["params"]
            params["page"] += 1
        url = QQNewsSpider.base_url + "?" + urlencode(params)
        yield response.follow(url, meta={"params": params}, callback=self.parse_re_dian_jing_xuan0)

    def parse_re_dian_jing_xuan0(self, response):
        list_info = ru.jsonp_to_json(response.text)
        if "data" in list_info.keys() and len(list_info["data"]) > 0:
            for req in self.download_re_dian_jing_xuan(response, response.meta["params"]["cid"]):
                yield req
        for req in self.parse_re_dian_jing_xuan(response):
            yield req

    def parse_re_dian_jing_xuan(self, response):
        list_info = ru.jsonp_to_json(response.text)
        if "data" in list_info.keys():
            for item in list_info["data"]:
                meta = dict()
                meta["list_info"] = item
                yield response.follow(url=item["vurl"], meta=meta, callback=self.parse_news_detail)
        else:
            logging.warning("Can not parse text: [url => %s ]" % response.url)

    def _match_new_url(self, url):
        if re.search(QQNewsSpider.re_news, url):
            return True

    def download_channels(self, response, cids):
        for cid in cids:
            for req in self.download_re_dian_jing_xuan(response, cid):
                yield req

    # def test_download_comments(self, response):
    #     response.meta["detail"] = {
    #             "comment_id": "4157351119",
    #     }
    #     for req in self.download_comments(response):
    #         yield req



import logging

from apscheduler.schedulers.blocking import BlockingScheduler
from scrapy.cmdline import execute

import os
import sys

from analysis.service import PublicOpinionService


class ModuleStarter:

    def __init__(self):
        self.scheduler = BlockingScheduler()
        self.service = PublicOpinionService()
        self.scheduler.add_job(self._execute_crawler, "interval", minutes=30)
        self.scheduler.add_job(self._analyze, "interval", minutes=30)

    def _analyze(self):
        self.service.update()

    def _execute_crawler(self):
        sys.path.append(os.path.abspath(__file__))
        execute(["scrapy", "crawl", "QQNewsSpider"])

    def start(self):
        self.scheduler.start()


if __name__ == '__main__':
    starter = ModuleStarter()
    starter.start()
    # starter._execute_crawler()
    # starter._analyze()



import jieba

for line in file.readlines():
    words = jieba.cut(line)
    for word in words:
        # print(word)
        if word not in stopword:
            output.write(word + " ")


def open_file():
    corpus = []
    token_path = "res_title_news.txt"
    try:

        with open(token_path, 'r', encoding="utf-8") as t:
            for line in t.readlines():
                corpus.append(line.strip())
        import gensim

        sentences = gensim.models.doc2vec.TaggedLineDocument(token_path)
        model = gensim.models.Doc2Vec(sentences, dm=1, size=200, window=10)
        model.train(sentences, total_examples=model.corpus_count, epochs=100)
        model.save('res_title_news_w2c.txt')
        out = open('res_title_news_vector.txt', 'w')
        for idx, docvec in enumerate(model.docvecs):
            for value in docvec:
                out.write(str(value) + ' ')
            out.write('\n')
    except Exception as e:
        print(e)


open_file()



from analysis.handle import ArticleHandle


class PublicOpinionService:
    def __init__(self):
        self.handler = ArticleHandle()

    def _service(self):
        self.handler.update_topic(100)
        self.handler.update_sentiment(100)

    def update(self):
        self._service()



import os
import time
import re
import jieba.analyse


def post_cut(url):
    fr = open(url + "/post_data.txt" ,"r" ,encoding="utf8")
    fo = open(url + "/post_key.txt", "a+" ,encoding="utf8")
    for line in fr.readlines():
        term = line.strip().split("\t")
        if len(term) == 3 and term[2] != "":
            key_list = jieba.analyse.extract_tags(term[2], 30)  # get keywords
            ustr = term[0] + "\t"
            for i in key_list:
                ustr += i + " "
            fo.write(ustr + "\n")
    fr.close()
    fo.close()


def post_tfidf(url):
    from sklearn.feature_extraction.text import HashingVectorizer
    fr = open(url + "/post_key.txt" ,"r" ,encoding="utf8")
    id_list = []
    data_list = []
    for line in fr.readlines():
        term = line.strip().split("\t")
        if len(term) == 2:
            id_list.append(term[0])
            data_list.append(term[1])

    hv = HashingVectorizer(n_features=10000, non_negative=True)  # 该类实现hash技巧
    post_tfidf = hv.fit_transform(data_list)  # return feature vector 'fea_train' [n_samples,n_features]
    print('Size of fea_train:' + repr(post_tfidf.shape))
    print(post_tfidf.nnz)
    post_cluster(url, id_list, post_tfidf)


def post_cluster(url, id, tfidf_vec):
    from sklearn.cluster import KMeans
    kmean = KMeans(n_clusters=300)
    print("kmeans" ,kmean.fit(tfidf_vec))
    #     pred = kmean.transform(tfidf_vec)

    count1 = 0
    count2 = 0
    #     pred_str = []
    #
    #     for item in pred:
    #         count1 += 1
    #         vec = ""
    #         for tmp in item :
    #             vec += str(tmp)[0:7] + "\t"
    #         pred_str.append(vec)
    #
    #     print len(pred_str)
    #     print len(id)

    pred = kmean.predict(tfidf_vec)
    fo = open(url + "/cluster.txt", "a+" ,encoding="utf8")
    for i in range(len(pred)):
        count2 += 1
        fo.write(id[i] + "\t" + str(pred[i]) + "\n")
    fo.close()
    print("%d+%d" % (count1, count2))


def post_lda(url, cluster):
    from gensim import corpora, models, matutils
    count = 0
    fr = open(url + "/post_key.txt" ,"r" ,encoding="utf8")
    fo2 = open(url + "/post_vec_lda.txt", "a+" ,encoding="utf8")
    id_list = []
    data_list = []

    for line in fr.readlines():
        term = line.strip().split("\t")
        if len(term) == 2:
            count += 1
            id_list.append(term[0])
            word = term[1].strip().split()
            data_list.append(word)
    print("lda")
    dic = corpora.Dictionary(data_list)  # 构造词典
    corpus = [dic.doc2bow(text) for text in data_list]  # 每个text 对应的稀疏向量
    tfidf = models.TfidfModel(corpus)  # 统计tfidf
    print("lda")
    corpus_tfidf = tfidf[corpus]  # 得到每个文本的tfidf向量，稀疏矩阵
    lda = models.LdaModel(corpus_tfidf, id2word=dic, num_topics=200)
    corpus_lda = lda[corpus_tfidf]  # 每个文本对应的LDA向量，稀疏的，元素值是隶属与对应序数类的权重
    print("lda")

    num = 0
    for doc in corpus_lda:
        wstr = ""
        for i in range(len(doc)):
            item = doc[i]
            wstr += str(item[0]) + "," + str(item[1])[0:7] + "/"
        fo2.write(id_list[num] + "\t" + wstr[0:-1] + "\n")
        num += 1
    fr.close()
    fo2.close()
    print(num)

    if cluster:
        lda_csc_matrix = matutils.corpus2csc(corpus_lda).transpose()  # gensim sparse matrix to scipy sparse matrix
        post_cluster(url, id_list, lda_csc_matrix)


if __name__ == "__main__":
    url = "../data"
    time = time.time()
    post_cut(url)
    post_tfidf(url)
    lda_cluster = False
    post_lda(url, lda_cluster)
    print(time.time() - time)


import time

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from crawler.settings import WEB_DRIVER


class ChromeBrowser:
    def __init__(self):
        chrome_options = Options()
        chrome_options.add_argument('--headless')
        chrome_options.add_argument('--disable-gpu')
        self.driver = webdriver.Chrome(chrome_options=chrome_options, executable_path=WEB_DRIVER)

    def close_web_driver(self):
        self.driver.quit()

    def get(self, request, crawler=None):
        d = self.driver
        d.get(request.url)


if __name__ == '__main__':
    driver = webdriver.Chrome(WEB_DRIVER)  # Optional argument, if not specified will search path.
    driver.get('http://www.google.com/')
    time.sleep(5)  # Let the user actually see something!
    search_box = driver.find_element_by_name('q')
    search_box.send_keys('ChromeDriver')
    search_box.submit()
    time.sleep(5)  # Let the user actually see something!
    driver.quit()



#!/usr/bin/env python
import setuptools
from setuptools import setup

setup(
    name='pubopinion',
    version='0.1.0a3',
    description='public opinion module',
    url='https://gitlab.com/dzywork/cadrem-pubopinion-crawler.git',
    author='戴志勇',
    author_email='dzywork@Outlook.com',
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords='crawler scrapy python',
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
)


# 从百度搜索引擎爬取数据
import logging
from collections import Iterable

import pymysql
import scrapy
from scrapy import Request
from scrapy_redis.spiders import RedisSpider

from crawler.items import HTMLItem

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


class BaiduSpider(RedisSpider):

    name = 'BaiduSpider'

    def start_requests(self):
        return self.get_starting_request(keywords=['阿里云'])

    def get_starting_request(self, keywords=None):
        if keywords is None:
            logger.debug("Keywords are None!")
            return None
        if not isinstance(keywords, Iterable):
            logger.warning("keywords aren't type of Iterable!")
            return None
        wd = ' '.join(keywords)
        search_url = 'https://www.baidu.com/s?ie=utf-8&wd=%s' % wd
        req = Request(url=search_url, callback=self.handle_baidu_page)
        return [req]

    def handle_baidu_page(self, response):
        # 百度搜索页链接抽取
        s_urls = response.xpath('//a/@href').re(r'/s\?.*')
        for su in s_urls:
            yield response.follow(url=su, callback=self.handle_baidu_page)
        i_urls = response.xpath('//*[@id="content_left"]//a/@href').getall()
        # 百度结果页链接抽取
        for iu in i_urls:
            yield response.follow(url=iu, callback=self.handle_specific_website)

    def handle_specific_website(self, response):
        # 保存
        item = HTMLItem()
        item['url'] = response.url
        item['text'] = response.text
        yield item

# if __name__ == '__main__':
#     bs = BaiduSpider()
#     req = bs.get_starting_request(['阿里云'])
#     print(req.url)


from collections import namedtuple

from crawler.db.db import StorageDAO
from crawler.nlpapi.baidu_aip import Aip, Constants
from crawler.utility.preprocesss import strip_tags


class TextAnalyzer(object):
    def __init__(self):
        self.aip = Aip()
        pass

    def topic(self, *articles):
        result = []
        if articles is None:
            return result
        for article in articles:
            result.append(self._topic_one(article))
        return result

    def _topic_one(self, article):
        return [self.aip.topic(title=article["title"], content=article["content"]), article]

    def _sentiment_classify_one(self, text):
        return [self.aip.sentiment_classify(text=text), text]

    def sentiment_classify(self, *texts):
        result = []
        if texts is None:
            return result
        for text in texts:
            result.append(self._sentiment_classify_one(text))
        return result


class Texts:
    analyzer = TextAnalyzer()

    def __init__(self):
        pass

    @staticmethod
    def sentiment(*texts):
        if texts is None or len(texts) < 1:
            return None
        results = Texts.analyzer.sentiment_classify(*texts)
        negative = 0  # 负向0
        normal = 0    # 中性1
        positive = 0  # 正向2
        for r, text in results:
            items = r["items"]
            for item in items:
                sentiment = item["sentiment"]
                if sentiment == Constants.SENTIMENT_NEGATIVE:
                    negative += 1
                elif sentiment == Constants.SENTIMENT_NORMAL:
                    normal += 1
                elif sentiment == Constants.SENTIMENT_POSITIVE:
                    positive += 1
        amounts = len(texts)
        return [negative/amounts, normal/amounts, positive/amounts]

    @staticmethod
    def topic(*articles):
        results = analyzer.topic(*articles)
        statistic = dict()
        for clazz in Constants.TOPIC_CLASS:
            statistic[clazz] = dict()
            statistic[clazz]["total"] = 0
            statistic[clazz]["articles"] = []
        for analysis_result, article in results:
            item = analysis_result["item"]
            lv1_tag_list = item["lv1_tag_list"]
            # lv2_tag_list = item["lv2_tag_list"]
            for lv1 in lv1_tag_list:
                if lv1["tag"] in statistic.keys():
                    statistic[lv1["tag"]]["articles"].append([analysis_result, article])
                    statistic[lv1["tag"]]["total"] += 1
        return statistic


class BaiduAnalyzer(object):
    def __init__(self):
        self.analyzer = Aip()

    def topic(self, *articles):
        """
        Classify for topics of articles
        :param articles: a list of mappings, for example:
            [{"id": id, "params": {"title": title, "content":content}}, ...]
        :return: A list of mappings that composed as [{"id": id, "result": result}, ...]
        """
        result = []
        for article in articles:
            params = article["params"]
            resp = self.analyzer.topic(title=params["title"], content=params["content"])
            result.append({"id": article["id"], "result": resp})
        return result

    def sentiment_classify(self, *texts):
        """
        Analyze sentiment of texts
        :param texts: A list of mappings, for example, [{"id": id, "params": {"text": text}}, ...]
        :return: A list of mappings, for example, ["id": id, "result": result]
        """
        result = []
        for text in texts:
            params = text["params"]
            resp = self.analyzer.sentiment_classify(text=params["text"])
            result.append({"id": text["id"], "result": resp})
        return result


if __name__ == '__main__':
    analyzer = TextAnalyzer()
    topic0 = {
        "title": "识别输入文本中有错误的片段",
        "content": "识别输入文本中有错误的片段，提示错误并给出正确的文本结果。支持短文本、长文本、语音等内容的错误识别，纠错是搜索引擎、语音识别、内容审查等功能更好运行的基础模块之一。",
    }
    topic1 = {
        "title": "识别输入文本中有错误的片段",
        "content": "识别输入文本中有错误的片段，提示错误并给出正确的文本结果。支持短文本、长文本、语音等内容的错误识别，纠错是搜索引擎、语音识别、内容审查等功能更好运行的基础模块之一。",
    }

    # result0 = analyzer.topic(topic0, topic1)
    # print(result0)
    text0 = "苹果是一家伟大的公司"
    text1 = "苹果是一家伟大的公司"
    # result1 = analyzer.sentiment_classify(text0, text1)
    # print(result1)
    # result3 = Texts.sentiment(topic0, topic1)
    # print(result3)
    # print(analyzer.sentiment_classify(text0))
    # result4 = Texts.topic(topic0, topic1)
    # print(result4)
    # dao = StorageDAO()
    # records = dao.select_named(1000, 100)
    # pages = [strip_tags(record.text) for record in records]
    # for page in pages:
    #     print(page+"\n")

    articles = [
        {
            "id": "1",
            "params": {
                "title": topic0["title"],
                "content": topic0["content"]
            }
        },
        {
            "id": "2",
            "params": {
                "title": topic1["title"],
                "content": topic1["content"],
            }
        }
    ]

    texts = [
        {
            "id": "1",
            "params": {
                "text": text0
            }
        },
        {
            "id": "1",
            "params": {
                "text": text1
            }
        }
    ]

    analyzer = BaiduAnalyzer()
    # print(analyzer.topic(*articles))
    # print(analyzer.sentiment_classify(*texts))



from crawler.db.db import get_connection

table_article = "cadrem_pubopinion_article"
table_comment = "cadrem_pubopinion_comment"


def select(table, offset=0, limit=100):
    """
    从表中选择limit条数据直到遍历结束
    :param table:
    :param offset:
    :param limit:
    :return: A tuple of tuples of rows
    """
    sql = " SELECT * FROM `%s` " % table
    sql += " LIMIT %(offset)s, %(limit)s "
    conn = get_connection()
    cursor = conn.cursor()
    is_remaining = True
    while is_remaining:
        params = {
            "offset": offset,
            "limit": limit,
        }
        cursor.execute(sql, params)
        sets = cursor.fetchall()
        if len(sets) > 0:
            offset += limit
            yield sets
        else:
            is_remaining = False
    conn.close()


def insert(table, fields, *rows):
    try:
        sql = "INSERT INTO `%(table)s`(%(fields)s)" \
              % {"table": table, "fields": ", ".join(["`"+field+"`" for field in fields])}
        # sql += ", ".join(["("+(", ".join(row))+")" for row in rows])
        conn = get_connection()
        cursor = conn.cursor()
        result = cursor.execute(sql, rows)
        cursor.close()
        conn.commit()
    finally:
        if conn:
            conn.close()
    return result


def update_sentiment_by_id(id, sentiment):
    sql = " UPDATE `%s` SET " % table_article
    sql += " sentiment=%s WHERE id = %s"
    conn = get_connection()
    cursor = conn.cursor()
    result = cursor.execute(sql, [sentiment, id])
    cursor.close()
    conn.commit()
    conn.close()
    return result


def update_by_id(*rows):
    """
    Update rows by id
    :param rows: A list of mapping that must has key named "id"
    :return:
    """
    if len(rows) < 1:
        return 0
    sql = " UPDATE `%s` SET " % table_article
    conn = get_connection()
    cursor = conn.cursor()
    sql += ", ".join(
        ["`"+field+"`=%("+field+")s" for row in rows for field in row.keys() if field != "id"]
    )
    sql += " WHERE id=%(id)s"
    result = cursor.executemany(sql, rows)
    cursor.close()
    conn.commit()
    conn.close()
    return result


def select_not_topic(table=table_article, batch=100):
    for rows in select(table, offset=0, limit=batch):
        selected = []
        for row in rows:
            if row[2] == "" or row[2] is None:
                selected.append(row)
        yield selected
    # yield [row for rows in select(table, offset=0, limit=batch) for row in rows if row[2] == "" or row[2] is None]


def select_not_sentiment(table=table_article, batch=100):
    for rows in select(table, offset=0, limit=batch):
        selected = []
        for row in rows:
            if row[9] == "" or row[9] is None:
                selected.append(row)
        yield selected
    # yield [row for rows in select(table, offset=0, limit=batch) for row in rows if row[9] == "" or row[9] is None]


if __name__ == '__main__':
    # result = update_sentiment_by_id("20190912V0GYF2", 0)
    # print(result)
    # rows = [
    #     {"id": "20190912V0GYF2", "sentiment": 1},
    #     {"id": "20190912V0GYF2", "sentiment": 2},
    #     {"id": "20190912V0GYF2", "sentiment": 3},
    #     {"id": "20190912V0GYF2", "sentiment": 4},
    # ]
    # result = update_by_id(rows)
    # print(result)
    for rows in select_not_topic():
        print(rows)
    for rows in select_not_sentiment():
        print(len(rows))


# -*- coding: utf-8 -*-
# filename: db.py
import threading
from collections import namedtuple

import math
import pymysql
import logging
from crawler.db.db_config import ENABLED_DB
from crawler.db.db_config import DB_CONFIG


class DatabaseConfigParser:

    logger = logging.getLogger("DatabaseConfigParser")

    def __init__(self, db_configs):
        if db_configs is None:
            raise BaseException("参数db_configs不能为空")

        self.db_configs = db_configs

    def get_config(self, config_name):
        logger = DatabaseConfigParser.logger
        configs = self.db_configs

        if config_name is None:
            logger.debug("config_name is None.")
            return

        for config in configs:
            if "name" in config.keys() and config["name"] == config_name:
                return config

        return


class DatabaseConfiguration(dict):
    logger = logging.getLogger("DatabaseConfiguration")

    def __init__(self, seq=None, **kwargs):
        self["ENABLED_DB"] = ENABLED_DB
        self.parser = DatabaseConfigParser(DB_CONFIG)
        self.__parse()

    def __parse(self):
        db_name = self["ENABLED_DB"]

        if db_name is None:
            self.logger.error("数据库配置错误，ENABLED_DB 为空")
            raise BaseException("database config exception, ENABLED_DB is None!")

        config = self.parser.get_config(db_name)

        if config is None:
            self.logger.error("数据库配置不存在！请检查DB_CONFIG配置项")
        else:
            for k in config:
                self[k] = config[k]


class MySqlHelper(object):

    logger = logging.getLogger("MySqlHelper")

    def __init__(self, db_config=None):
        self.config = DatabaseConfiguration() if not db_config else db_config
        self.conn = self.__connect_db()
        if "pool_size" not in self.config.keys() or type(self.config["pool_size"]) != "int":
            self.pool_size = 5
        else:
            self.pool_size = self.config["pool_size"]
        self.pool = []
        # 初始化线程池
        for i in range(self.pool_size):
            self.pool.append(self.__connect_db())

    def __is_connect(self):
        self.logger.debug("数据库连接中……")
        config = self.config

        if not config:
            self.logger.debug("获取数据库配置失败！")
        conn = pymysql.connect(config["hostname"], config["username"],
                               config["password"])

        if conn is None:
            self.logger.error("数据库连接失败！")
            return False

        if config["database"] is None:
            self.logger.error("数据库名不可为空。")
            return False

        # 创建数据库
        cursor = conn.cursor()
        sql = "CREATE DATABASE IF NOT EXISTS `%s` DEFAULT CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci;" \
              % config["database"]
        cursor.execute(sql)
        cursor.close()
        conn.close()
        return True

    def __connect_db(self):
        config = self.config
        if self.__is_connect():
            return pymysql.connect(config["hostname"], config["username"],
                                   config["password"], config["database"])

    def get_one(self, sql, param=None):
        conn = self.__get_connection()
        cursor = conn.cursor()
        cursor.execute(sql, param)
        data = cursor.fetchone()
        cursor.close()
        self.__release(conn)
        return data

    def get_all(self, sql, param=None):
        conn = self.__get_connection()
        cursor = conn.cursor()
        cursor.execute(sql, param)
        data = cursor.fetchall()
        cursor.close()
        self.__release(conn)
        return data

    def get_dict(self, sql, param=None):
        return self.get_all(sql, param)

    def __release(self, conn):
        if conn is not None:
            self.pool.append(conn)

    def __get_connection(self):
        conn = None
        while conn is None:
            if len(self.pool) > 0:
                conn = self.pool.pop()
        return conn

    def insert(self, sql, data=None):
        conn = self.__get_connection()
        cursor = conn.cursor()
        row_count = cursor.execute(sql, data)
        conn.commit()
        cursor.close()
        self.__release(conn)
        return row_count

    def insert_many(self, sql, data=None):
        conn = self.__get_connection()
        cursor = conn.cursor()
        row_count = cursor.execute(sql, data)
        conn.commit()
        cursor.close()
        self.__release(conn)
        return row_count

    def delete(self, sql, param=None):
        conn = self.__get_connection()
        cursor = conn.cursor()
        row_count = cursor.execute(sql, param)
        conn.commit()
        cursor.close()
        self.__release(conn)
        return row_count

    def update(self, sql, param=None):
        conn = self.__get_connection()
        cursor = conn.cursor()
        row_count = cursor.execute(sql, param)
        conn.commit()
        cursor.close()
        self.__release(conn)
        return row_count

    def execute_batch(self, sql, param=None):
        conn = self.__get_connection()
        cursor = conn.cursor()
        row_count = cursor.executemany(sql, param)
        conn.commit()
        cursor.close()
        self.__release(conn)
        return row_count

    def close(self):
        pool_size = self.pool_size
        while pool_size > 0:
            conn = self.__get_connection()
            pool_size -= 1
            conn.close()


class SingleMySqlPool(MySqlHelper):
    count = 0
    self = None
    lock = threading.Lock()

    @staticmethod
    def get_instance():
        lock = SingleMySqlPool.lock
        if SingleMySqlPool.self is None:
            lock.acquire()
            if SingleMySqlPool.self is None:
                SingleMySqlPool.self = SingleMySqlPool()
            lock.release()
        lock.acquire()
        SingleMySqlPool.count += 1
        lock.release()
        return SingleMySqlPool.self

    def close(self):
        lock = SingleMySqlPool.lock
        lock.acquire()
        SingleMySqlPool.count -= 1
        lock.release()
        if SingleMySqlPool.count == 0:
            super().close()


def get_connection():
    config = {
        "name": "test",
        "hostname": "localhost",
        "username": "root",
        "password": "123",
        "database": "public_opinion_test0",
        "pool_size": 5
    }
    conn = pymysql.connect(config["hostname"], config["username"],
                           config["password"], config["database"])
    return conn


class StorageDAO(object):

    MAX_LIMIT = 2147483648

    def __init__(self):
        pass

    def insert(self):
        conn = get_connection()
        cursor = conn.cursor()
        # cursor.execute(query="")
        cursor.close()
        conn.commit()
        conn.close()

    def select(self, offset=0, limit=MAX_LIMIT):
        conn = get_connection()
        cursor = conn.cursor()
        args = {
            "offset": offset,
            "limit": limit,
        }
        cursor.execute("SELECT * FROM `text` LIMIT %(offset)s, %(limit)s", args)
        result = cursor.fetchall()
        cursor.close()
        conn.close()
        return result

    def select_named(self, offset=0, limit=MAX_LIMIT):
        result = self.select(offset, limit)
        if len(result) < 0:
            return result
        HTMLEntity = namedtuple("HTMLEntity", ["url", "text"])
        result_set = []
        for record in result:
            e = HTMLEntity(*record)
            result_set.append(e)
        return result_set
